/*===========================================================================*
 * biaxis.c                                                                  *
 *                                                                           *
 *      Procedures for calculating conoscopic light intensity                *
 *      (Biaxial Conoscopic Figure)                                          *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void init_biaxial(double tha, double pha, double thb, double phb,    *
 *                        double alpha, double gamma, double d)              *
 *      void setup_biaxial(double th, double ph)                             *
 *      double get_biaxial_retardation()                                     *
 *      double get_biaxial_diagonal_degree()                                 *
 *                                                                           *
 *      (In these procedures angles are expressed as radian)                 *
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#include <math.h>


/*==================*
 * STATIC VARIABLES *
 *==================*/
static double RI2;                  /* alpha*gamma */
static double RI2Add;               /* alpha*alpha + gamma*gamma */
static double RI2Sub;               /* gamma*gamma - alpha*alpha */
static double RayLength;            /* Ray length passing through crystal */
static double thickness;

static double xa, ya, za;           /* unit vector component of A axis */ 
static double xb, yb, zb;           /* unit vector component of B axis */

static double cos_rtha, sin_rtha;          
static double cos_rthb, sin_rthb;
static double cos_rpha, sin_rpha;
static double cos_rphb, sin_rphb;

/*====================* 
 * Internal Functions *
 *====================*/
static void rotate_optic_axes(double th, double ph);

/*===========================================================================*
 * (Exported Procedure)                                                      *
 *                                                                           *
 *            init_biaxial                                                   *
 *               (Initial setting before calculating conoscopic figure)      *
 *===========================================================================*/
void 
init_biaxial(double tha, double pha, double thb, double phb, 
	    double alpha, double gamma, double d)
{
   double sin_tha, cos_tha;     /* orientations of optic axes */
   double sin_pha, cos_pha;
   double sin_thb, cos_thb;     
   double sin_phb, cos_phb;

   sin_tha = sin(tha); cos_tha = cos(tha);    
   sin_pha = sin(pha); cos_pha = cos(pha); 
 
   sin_thb = sin(thb); cos_thb = cos(thb);    
   sin_phb = sin(phb); cos_phb = cos(phb); 
   
   xa = sin_tha*cos_pha;  ya = sin_tha*sin_pha;  za = cos_tha;
   xb = sin_thb*cos_phb;  yb = sin_thb*sin_phb;  zb = cos_thb;


   RI2 = alpha*gamma; 
   RI2Add = alpha*alpha + gamma*gamma;    
   RI2Sub = gamma*gamma - alpha*alpha;

   thickness = d;
}

 /*==========================================================================*
 * (Exported Procedure)                                                      *
 *                                                                           *
 *            setup_biaxial                                                  *
 *               (setting before calculating each ray interderence color     *
 *===========================================================================*/
void 
setup_biaxial(double th, double ph)
{
   rotate_optic_axes(th, ph);
   RayLength = thickness/cos(th);
}

/*==========================================================================*
* (Exported Procedure)                                                      *
*                                                                           *
*                   get_biaxial_retardation                                 *
*===========================================================================*/
double 
get_biaxial_retardation()
{
   double ri1 
     = RI2*sqrt(2.0/(RI2Add + RI2Sub*(cos_rtha*cos_rthb+sin_rtha*sin_rthb)));
   double ri2 
     = RI2*sqrt(2.0/(RI2Add + RI2Sub*(cos_rtha*cos_rthb-sin_rtha*sin_rthb)));

   return  RayLength* (ri2 - ri1);
}

/*==========================================================================*
* (Exported Procedure)                                                      *
*                                                                           *
*                   get_biaxial_diagonal_degree                             *
*===========================================================================*/
double 
get_biaxial_diagonal_degree()
{
   return sin_rpha*cos_rphb + sin_rphb*cos_rpha;
}

/*==========================================================================*
* (Internal Procedure)                                                      *
*                                                                           *
*                   rotate_optic_axes                                       *
*===========================================================================*/
static void 
rotate_optic_axes(double th, double ph) 
{
   double cos_th = cos(th);
   double sin_th = sin(th);
   double cos_ph = cos(ph);
   double sin_ph = sin(ph);

   double rxa, rya, rza;        /* unit vector component of rotated A */ 
   double rxb, ryb, rzb;        /* unit vector component of rotated B */

   double k00, k10, k11, k20, k21, k22;

   k00 = cos_ph*cos_ph*cos_th + sin_ph*sin_ph;
   k10 = sin_ph*cos_ph*(cos_th-1);
   k11 = sin_ph*sin_ph*cos_th + cos_ph*cos_ph;
   k20 = cos_ph*sin_th;
   k21 = sin_ph*sin_th;
   k22 = cos_th;

   rxa = k00*xa + k10*ya - k20*za;
   rya = k10*xa + k11*ya - k21*za;
   rza = k20*xa + k21*ya + k22*za;

   rxb = k00*xb + k10*yb - k20*zb;
   ryb = k10*xb + k11*yb - k21*zb;
   rzb = k20*xb + k21*yb + k22*zb;

   cos_rtha = rza;
   cos_rthb = rzb;

   sin_rtha = sqrt(1 - rza*rza);
   sin_rthb = sqrt(1 - rzb*rzb);
   
   cos_rpha = rxa/sin_rtha;
   cos_rphb = rxb/sin_rthb;
   sin_rpha = rya/sin_rtha;
   sin_rphb = ryb/sin_rthb;
}

