/*===========================================================================*
 * conoscopic_RGB.c                                                          *
 *                                                                           *
 *      Calculating conoscopic RGB DATA in RGB array (float)                 *
 *            if out of orientation range, -1.0 is stored                    *
 *            else the values between 0.0 and 1.0 are stored in the array    *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      float get_conoscopic_RGB(float *RGB)                                 *
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#include <stdio.h>
#include <math.h>
#include "cono_map.h"

/*==================*
 * GLOBAL VARIABLES *
 *==================*/
extern gfloat RI, NA;
extern gint image_size;
extern gfloat image_radius;

/*==================*
 * STATIC VARIABLES *
 *==================*/
static double image_radius_90;   /* image_radius for max_th = 90 deg*/

/*====================*
 * EXTERNAL FUNCTIONS *
 *====================*/
extern void init_uniaxial(double thc, double phc,                           
                          double omega, double epsilon, double d);  
extern void init_biaxial(double tha, double pha, double thb, double phb, 
			double alpha, double gamma, double d);

extern void setup_uniaxial(double th, double ph);
extern void setup_biaxial(double th, double ph);

extern double get_uniaxial_retardation();
extern double get_biaxial_retardation();

extern double get_uniaxial_diagonal_degree();
extern double get_biaxial_diagonal_degree();
 
extern void get_interference_color_RGB(double R, double DD, float* RGB);
extern void get_interference_color_RGB_with_testplate(double R, double DD, 
						      float* RGB); 
extern double (*get_th)(double dis);
extern double (*get_dis)(double th);

/*======================*
 * POINTTER TO FUNCTION *
 *======================*/
void (*setup_orientation)(double, double);
void (*get_interference_RGB)(double, double, float*);
double (*get_retardation)();
double (*get_diagonal_degree)();

/*====================*
 * INTERNAL PROCEDURE *
 *====================*/
static void setup();

/*===========================================================================*
 * (EXPORTED FUNCTION)                                                       *
 *                                                                           *
 *               get_conoscopic_RGB                                          *
 *      Setting RGB data between 0 and 1 and returning maximum RGB value     *
 *===========================================================================*/
float get_conoscopic_RGB(float *RGB)
{
   double ph, th;       /* Orientation in Conoscopic Figure */
   int i, j, k;
   double x, y, length_from_center;
   float max_RGB;

   setup();
   
   max_RGB = 0;
   for( k = 0; k < image_size*image_size; k++)
   {
      i = k % image_size;
      j = k / image_size;
      x = i - (image_radius-0.5);
      y = (image_radius-0.5) - j;

      length_from_center = sqrt(x*x + y*y);
      if( length_from_center > image_radius )
      {
	 RGB[3*k] = RGB[3*k+1] = RGB[3*k+2] = -1;
	 continue;
      }

      if( drawing_method.drawing_range==WholeAngle)
	th = get_th( length_from_center/image_radius);
      else
      {
	 if (drawing_method.projection == RealProjection)
	   th = asin((length_from_center/image_radius_90)/RI);
	 else 
	   th = asin(sin(get_th(length_from_center/image_radius_90)/RI) );
      }
      
      ph = ( length_from_center != 0 ) ? atan(y/x) : M_PI/2;
      if(x < 0) ph += M_PI;
      
      setup_orientation(th, ph);

      get_interference_RGB( (*get_retardation)(), 
			    (*get_diagonal_degree)(), &(RGB[3*k]));
      
      if(RGB[3*k] > max_RGB)  max_RGB = RGB[3*k]; 
      if(RGB[3*k+1] > max_RGB)  max_RGB = RGB[3*k+1]; 
      if(RGB[3*k+2] > max_RGB)  max_RGB = RGB[3*k+2]; 
   }
   return max_RGB;
}

/*===========================================================================*
 * (INTERNAL PROCEDURE)                                                       *
 *                                                                           *
 *                       setup                                               *
 *===========================================================================*/
static void setup()
{
   double d;                 /* thickness of thin-section */ 

   d = parameter.thickness * 1000;   

   if(drawing_method.axial_type == Uniaxial)
   {
      setup_orientation = &setup_uniaxial;
      get_retardation = &get_uniaxial_retardation;
      get_diagonal_degree = &get_uniaxial_diagonal_degree;
   }
   else
   {
      setup_orientation = &setup_biaxial;
      get_retardation = &get_biaxial_retardation;
      get_diagonal_degree = &get_biaxial_diagonal_degree;
   }


   if(drawing_method.test_plate == Out)
     get_interference_RGB = &get_interference_color_RGB;
   else	   
     get_interference_RGB = &get_interference_color_RGB_with_testplate;

   if(drawing_method.drawing_range ==WithinOpeningAngle)
     image_radius_90 = image_radius/get_dis(asin(NA));

   if(drawing_method.axial_type == Uniaxial)
     init_uniaxial(Caxis.th, Caxis.ph, parameter.omega, parameter.epsilon, d);
   else /*that is, for Biaxial */
     init_biaxial(Aaxis.th, Aaxis.ph, Baxis.th, Baxis.ph,
		  parameter.alpha, parameter.gamma, d);
}


