/*===========================================================================*
 * interference_color.c                                                      *
 *                                                                           *
 *      Procedure for obtaining interference color RGB                       *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void get_interference_color_RGB(double Ret, double DD, float* RGB)   *
 *      void get_interference_color_RGB_with_testplate                       *
 *                                     (double Ret, double DD, float* RGB)   *
 *                                                                           *
 *                          Ret: retardation                                 *
 *                          DD: diagonal degree                              *
 *                            (0:extinction position, +-1:diagonal position) *
 *                                                                           *
 *                                                            (by A. Miyake) *
 *===========================================================================*/

#include <math.h>
#define MAX_R (10000)         /* precalculating RGB DATA upto 10000 nm */

/*==================*
 * STATIC VARIABLES *
 *==================*/
static double R;               /* temporal retardation */
static double  tR = 530;       /* testplate retardation*/
static float RGB_Table[3*MAX_R];
static int create_RGB_Table = 0;
 
/*===================*
 * EXTERNAL FUNCTION *
 *===================*/
extern void get_RGB(double (*get_I)(double), float* RGB );

/*====================*
 * INTERNAL PROCEDURE *
 *====================*/
static void setup_interference_color_table();
static double get_std_I(double lambda);
 
/*===========================================================================*
 * (EXPOETED PROCEDURE)                                                      *
 *                     get_interference_color_RGB                            *
 *===========================================================================*/
void 
get_interference_color_RGB(double Ret, double DD, float* RGB)
{
   int k;
   double RR = fabs(Ret);
   int low = floor(RR);
   int high = low+1;

   if(!create_RGB_Table)
     setup_interference_color_table();

   if( low < MAX_R - 1)
   {
      for(k = 0; k < 3; k++)
      {
	 RGB[k] = (high-RR)*RGB_Table[3*low+k]+(RR-low)*RGB_Table[3*high+k];
      }
   }
   else
   {
      R = RR; get_RGB(&get_std_I, RGB);
   }
   for(k = 0; k < 3; k++)
   {
      RGB[k] *= DD*DD;
      if(RGB[k] > 1) RGB[k] = 1;
      if(RGB[k] < 0) RGB[k] = 0;
   }
}
      

/*===========================================================================*
 * (EXPOETED PROCEDURE)                                                      *
 *                     get_interference_color_RGB_with_testplate             *
 *===========================================================================*/
void 
get_interference_color_RGB_with_testplate(double Ret, double DD, float* RGB)
{
   int k, i;
   double RR[3];
   int low, high;
   static float I[3][3], deg[3];

   if(!create_RGB_Table)
     setup_interference_color_table();

   RR[0] = fabs(Ret+tR); RR[1] = fabs(Ret-tR); RR[2] = tR;
   deg[0] = (DD*DD + DD)/2; deg[1] = (DD*DD - DD)/2; deg[2] = 1 - DD*DD;

   for(i = 0; i < 3; i++)
   {
     if( RR[i] < MAX_R - 1)
      {
	 low = floor(RR[i]);
	 high = low + 1;

 	  for(k = 0; k < 3; k++)
	  {
	     I[i][k] = (high-RR[i])*RGB_Table[3*low+k]
	             + (RR[i]-low)*RGB_Table[3*high+k];
	  }
      }
      else
      {
	R = RR[i]; get_RGB(&get_std_I, I[i]);
      }
   }
   for(k = 0; k < 3; k++)
   { 
      RGB[k] = 0;
      for(i = 0; i < 3; i++)
	RGB[k] += deg[i]*I[i][k];

      if(RGB[k] > 1) RGB[k] = 1;
      if(RGB[k] < 0) RGB[k] = 0;
   }
}

/*===========================================================================*
 * (INTERNAL PROCEDURES)                                                     *
 *                     get_std_I   ( Before calling this, R must be set)     *
 *                     setup_interference_color_table                        *
 *===========================================================================*/
static double 
get_std_I(double lambda)
{
   double s = sin(M_PI*R/lambda);
   return s*s;
}
   
static void
setup_interference_color_table() 
{
   int i;
   
   for(i = 0; i < MAX_R; i++)
   {
      R = i; get_RGB( &get_std_I, RGB_Table+3*i );
   }
   create_RGB_Table = 1;
}

      
    
   
