/*===========================================================================*
 * orientation.c                                                             *
 *                                                                           *
 *      Procedures to create or update Orientation Frame                     *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void createOrientationFrame(GtkWidget *)                             *
 *      void updateOrientationFrame()                                        * 
 *      void changeOpticAngle()                                              *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <gtk/gtk.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "cono_map.h"
#include "widget_set.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "intl.h"

/*==================*
 * GLOBAL VARIABLES *   
 *==================*/
extern GtkStyle* GreekStyle;
extern GtkTooltips *tooltips;

/*==================*
 * GLOBAL FUNCTIONS *   
 *==================*/
extern void updateStereoFrame();
extern void updateOtherParametFrame();

/*==================*
 * STATIC VARIABLES *
 *==================*/
static GtkWidget *OrientationFrame;
static GtkWidget *OrientationBox = NULL;

/*=========================* 
 * Main Internal Functions *
 *=========================*/
static void SamplingAzimuthStep(), UniaxialOrientation(), BiaxialOrientation();
static void setParamet(GtkSpinButton*, gfloat*);


/*=====================*
 * EXPORTED PROCEDURES *
 *=====================*/
void createOrientationFrame(GtkWidget *);
void updateOrientationFrame();
void changeOpticAngle();  /* only for biaxial materials */

/*===============================================================*
 * (EXPORTED PROCEDURE)                                          *
 *               creatueOrientationFrame                         *
 *===============================================================*/
void createOrientationFrame(GtkWidget *parent)
{
   OrientationFrame = gtk_frame_new(_("(Crystal Orientation)"));
   gtk_container_set_border_width( GTK_CONTAINER(OrientationFrame), 10);
   gtk_box_pack_start(GTK_BOX(parent), OrientationFrame, FALSE, FALSE, 0);

   OrientationBox  = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(OrientationFrame), OrientationBox);
   
   updateOrientationFrame();
}

/*===============================================================*
 * (EXPORTED PROCEDURE)                                          *
 *                  updateOrientationFrame                       *
 *===============================================================*/
void updateOrientationFrame()
{
   if(OrientationBox != NULL)
     gtk_widget_destroy(OrientationBox);

   OrientationBox  = gtk_vbox_new(FALSE, 0);
   gtk_container_add(GTK_CONTAINER(OrientationFrame), OrientationBox);
      
   if(drawing_method.axial_type == Uniaxial)
     UniaxialOrientation();
   else if(drawing_method.axial_type == Biaxial)
     BiaxialOrientation();

   if(drawing_method.drawing_kind != Single)
     SamplingAzimuthStep();

   gtk_widget_show_all(OrientationBox);
}   



/*===============================================================*
 * (Internal Procedure)                                          *
 *                    SamplingAzimuthStep                        *
 *===============================================================*/
static void setSamplingStep(GtkSpinButton*, gfloat*);

static void  SamplingAzimuthStep()
{
   gfloat step=0, min_step=0, max_step=0, *sampling_step_ptr=NULL;
   GtkWidget *SamplingStepSpin, *spin_label;
   gfloat tic = 0;
   gint digit = 0;

   if(drawing_method.drawing_kind == Circular)
   {
      step = parameter.sampling_step = parameter.sampling_step_for_Circular;
      min_step = 15;
      max_step = 45;
      tic = 0.5;
      digit = 1;
      sampling_step_ptr = &(parameter.sampling_step_for_Circular);
   }
   else if(drawing_method.drawing_kind == Animation)
   {
      step = parameter.sampling_step = parameter.sampling_step_for_Animation;
      min_step = 1;
      max_step = 10;
      tic = 1;
      digit = 0;
      sampling_step_ptr = &(parameter.sampling_step_for_Animation);
   }

   SamplingStepSpin = pack_spin(OrientationBox, ("Df  "), 160,
				50, step, min_step, max_step, 
				tic, tic, digit);
   spin_label =  gtk_object_get_data(GTK_OBJECT(SamplingStepSpin),"label");
   gtk_widget_set_style( spin_label, GreekStyle);   
   gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON(SamplingStepSpin),TRUE); 
   gtk_tooltips_set_tip (tooltips, SamplingStepSpin, 
			 _("Azimuth Sampling Step"), NULL);

   gtk_signal_connect(GTK_OBJECT (SamplingStepSpin), "changed",
		      GTK_SIGNAL_FUNC (setSamplingStep), sampling_step_ptr);
}

static void
setSamplingStep(GtkSpinButton* spin, gfloat* var) 
{
   gint pre_val, val;

   if(drawing_method.drawing_kind == Circular)
   {
      pre_val = parameter.sampling_step*2;
      setParamet(spin, var);
      val = *var*2;
      while(360 % val)
      {
	 if(val > pre_val) val++;
	 else              val--;
      }
      gtk_spin_button_set_value(spin, val/2.0);
 
      parameter.sampling_step = parameter.sampling_step_for_Circular;
   }
   else if(drawing_method.drawing_kind == Animation)
   {
      setParamet(spin, var);
      parameter.sampling_step = parameter.sampling_step_for_Animation;
   }
   updateOtherParametFrame();
}

/*===============================================================*
 * (Internal Procedure)                                          *
 *                    setParamet                                 *
 *===============================================================*/
static void
setParamet(GtkSpinButton* spin, gfloat* paramet) 
{
   *paramet = gtk_spin_button_get_value_as_float(spin);
}

/*===============================================================*
 * (Internal Procedure)                                          *
 *                    UniaxialOrientation                        *
 *===============================================================*/

static void changeC(GtkSpinButton* spin, gfloat* paramet);

void
UniaxialOrientation()
{	    

   GtkWidget *CLabel, *CThSpin, *CPhSpin;
   GtkWidget *CAxisBox = gtk_hbox_new(FALSE, 0);

   pack_label(OrientationBox, _("C : Optic Axis"), 10);

   gtk_box_pack_start (GTK_BOX(OrientationBox), CAxisBox, FALSE, FALSE, 10);

   CLabel = gtk_label_new("C");
   gtk_widget_set_usize (CLabel, 53, 0);
   gtk_box_pack_start (GTK_BOX(CAxisBox), CLabel, FALSE, FALSE, 0);
    
   CThSpin = pack_spin( CAxisBox, "q  ", 25, 50, parameter.c_theta, 
		       0, 90, 1, 10, 0);
   gtk_tooltips_set_tip (tooltips, CThSpin, _("Vertical Angle"), NULL);

   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(CThSpin), "label"),
			GreekStyle);   

   gtk_signal_connect(GTK_OBJECT(CThSpin), "changed", 
		      GTK_SIGNAL_FUNC(changeC), &(parameter.c_theta));  
 
//   if(drawing_method.drawing_kind == Single)
   {
      CPhSpin = pack_spin( CAxisBox, "f  ", 25, 50, parameter.c_phi, 
		    0, 360, 1, 10, 0);
      gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(CPhSpin), TRUE);
      gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(CPhSpin), "label"),
			GreekStyle);   
      gtk_tooltips_set_tip (tooltips, CPhSpin, _("Azimuth"), NULL);
      gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(CPhSpin), TRUE);
      
      gtk_signal_connect(GTK_OBJECT(CPhSpin), "changed", 
		      GTK_SIGNAL_FUNC(changeC), &(parameter.c_phi));  
   }
}   

static void    
changeC(GtkSpinButton* spin, gfloat* paramet)
{
   setParamet(spin, paramet);
   updateStereoFrame();
}   


/*===============================================================*
 * (Internal Procedure)                                          *
 *                    BiaxialOrientation                        *
 *===============================================================*/

typedef struct 
{
   gchar name[2];  
   GtkWidget *box; 
   gfloat *th, *ph, pre_th, pre_ph;
} Axis;

/*========= Static Variables for BiaxialOrientation =====================*/
static Axis A = { "A", NULL, &(parameter.a_theta), &(parameter.a_phi), 0, 0 };
static Axis B = { "B", NULL, &(parameter.b_theta), &(parameter.b_phi), 0, 0 };
static Axis X = { "X", NULL, &(parameter.x_theta), &(parameter.x_phi), 0, 0 };
static Axis Y = { "Y", NULL, &(parameter.y_theta), &(parameter.y_phi), 0, 0 };
static Axis Z = { "Z", NULL, &(parameter.z_theta), &(parameter.z_phi), 0, 0 };
static Axis *AllAxes[5] = {&A, &B, &X, &Y, &Z};
static Axis *PriorAxis = &A;
static GtkWidget *PriorThSpin, *PriorPhSpin, *RotSpin;
static gboolean YSide = TRUE; /* TRUE if front side and FALSE if back side */

/*========== Internal Procedures for BiaxialOrientation ==================*/
static void createPriorBox(Axis *);
static void createNonPriorBox(Axis *);
static void changePriorTh(GtkSpinButton*, Axis *); 
static void changePriorPh(GtkSpinButton*,Axis *);
static void rotateAroundPriorAxes(GtkSpinButton*, Axis *);
static void PriorAxisChange(GtkRadioButton *, Axis *);
static gboolean rotate(gfloat, gfloat, gfloat*, gfloat*, gfloat[][3]);
static void get_rotate_matrix(gfloat, gfloat, gfloat, gfloat[][3]);

void
BiaxialOrientation()
{
   GtkWidget *AxisLegendBox;
   GSList* axis_group = NULL;
   GtkWidget *axis_hbox[5];
   GtkWidget *AxisRadio[5];
   gint i;

   AxisLegendBox = gtk_vbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(OrientationBox), AxisLegendBox, FALSE, FALSE,10);

   pack_label( AxisLegendBox, _("A B : Optic Axes"), 0 );
   pack_label( AxisLegendBox, _("X Y Z : Elasticity Axes"), 0 );
 
   for(i = 0; i < 5; i++)
   {
      axis_hbox[i] = gtk_hbox_new(FALSE, 0);
      gtk_box_pack_start (GTK_BOX(OrientationBox), axis_hbox[i], 
		       FALSE, FALSE, 0);
      
      AxisRadio[i]
	= gtk_radio_button_new_with_label(axis_group, AllAxes[i]->name);
      axis_group = gtk_radio_button_group (GTK_RADIO_BUTTON (AxisRadio[i]));
      gtk_box_pack_start(GTK_BOX (axis_hbox[i]), AxisRadio[i], FALSE,FALSE,10);

      AllAxes[i]->box = NULL;
      if(AllAxes[i] == PriorAxis)
	createPriorBox(PriorAxis);
      else
	createNonPriorBox(AllAxes[i]);

      gtk_box_pack_start(GTK_BOX(axis_hbox[i]), AllAxes[i]->box,FALSE,FALSE,0);

      gtk_signal_connect(GTK_OBJECT(AxisRadio[i]), "toggled",
			   GTK_SIGNAL_FUNC(PriorAxisChange), AllAxes[i]);
   }
   for(i = 0; i < 5; i++)
   {
      if(AllAxes[i] == PriorAxis)
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (AxisRadio[i]), TRUE);
   }
}


void
createPriorBox(Axis *axis)
{
   
   if(axis->box != NULL) gtk_widget_destroy(axis->box);

   axis->box = gtk_hbox_new(FALSE, 0);
   PriorThSpin = pack_spin(axis->box, "q  ", 25, 50,*(axis->th), 0, 90,1,10,0);
   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(PriorThSpin), "label"),
			GreekStyle);   
   gtk_tooltips_set_tip (tooltips, PriorThSpin, _("Vertical Angle"), NULL);

   PriorPhSpin =pack_spin(axis->box, "f  ", 25, 50, *(axis->ph), 0,360,1,10,0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(PriorPhSpin), TRUE);
   gtk_widget_set_style(gtk_object_get_data(GTK_OBJECT(PriorPhSpin), "label"),
			GreekStyle);   
   gtk_tooltips_set_tip (tooltips, PriorPhSpin, _("Azimuth"), NULL);

   RotSpin = pack_spin(axis->box, "R  ", 20, 16, 0, -180, 180, 1, 10, 0);
   gtk_spin_button_set_wrap (GTK_SPIN_BUTTON(RotSpin), TRUE);
   gtk_tooltips_set_tip (tooltips, RotSpin, 
			 _("Rotation around this axis"), NULL);

   
   gtk_signal_connect(GTK_OBJECT(PriorThSpin), "changed",
			     GTK_SIGNAL_FUNC(changePriorTh), axis);
   gtk_signal_connect(GTK_OBJECT(PriorPhSpin), "changed",
			      GTK_SIGNAL_FUNC(changePriorPh), axis);
   gtk_signal_connect(GTK_OBJECT(RotSpin), "changed",
			     GTK_SIGNAL_FUNC(rotateAroundPriorAxes), axis);
   gtk_widget_show_all(axis->box);
}

void
createNonPriorBox(Axis *axis )
{
   GtkWidget *th_label, *ph_label, *th_val, *ph_val;
   char str[100];
   
   if(axis->box != NULL) gtk_widget_destroy(axis->box);

   axis->box = gtk_hbox_new(FALSE, 0);
 
   th_label = pack_label( axis->box, "q  ", 0);
   gtk_widget_set_style(th_label, GreekStyle);
   gtk_misc_set_alignment(GTK_MISC(th_label), 1, 0.5);
   gtk_widget_set_usize (th_label, 27, 0);

   sprintf(str, "%3.0f", *(axis->th));
   th_val = pack_label(axis->box, str, 0);
   gtk_misc_set_alignment(GTK_MISC(th_val), 0, 0.5);
   gtk_widget_set_usize (th_val, 50, 0);
   
   ph_label = pack_label( axis->box, "f  ", 0);
   gtk_widget_set_style(ph_label, GreekStyle);
   gtk_misc_set_alignment(GTK_MISC(ph_label), 1, 0.5);
   gtk_widget_set_usize (ph_label, 30, 0);

   sprintf(str, "%3.0f", *(axis->ph));
   ph_val = pack_label(axis->box, str, 0);
   gtk_misc_set_alignment(GTK_MISC(ph_val), 0, 0.5);
   gtk_widget_set_usize (ph_val, 50, 0);
   
   gtk_widget_show_all(axis->box);
}

/*======================================================================*
* Callback Functions for BiaxialOrientation                             *
*                                                                       *
*                      changePriorTh                                    *
*                      changePriorPh                                    *
*                      rotateAroundPriorAxes                            *
*                      PriorAxisChange                                  *
*=======================================================================*/
static enum {None, ChangeTh, ChangePh, Rotation} LastChange = None;

static void 
changePriorTh(GtkSpinButton *spin, Axis *axis)
{
   gfloat mat[3][3], dth;
   gint i;
   gboolean normal;
 
   if(LastChange == Rotation)
   {
      LastChange = None;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(RotSpin), 0);
   }
   
    if(LastChange != ChangeTh)
    {
      for(i = 0; i < 5; i++)
      {
	 AllAxes[i]->pre_th = *(AllAxes[i]->th);
	 AllAxes[i]->pre_ph = *(AllAxes[i]->ph);
      }
      LastChange = ChangeTh;
   }    
   
   setParamet(spin, axis->th);
   dth = *(axis->th) - axis->pre_th;
   get_rotate_matrix(90, axis->pre_ph+90, dth, mat);

   for(i = 0; i < 5; i++)
   {
      if(AllAxes[i] != axis)
      {
	 GtkWidget *parent = AllAxes[i]->box->parent;

	 normal = rotate(AllAxes[i]->pre_th, AllAxes[i]->pre_ph, 
			 AllAxes[i]->th, AllAxes[i]->ph, mat);
	 
	 if(AllAxes[i] == &Y && normal == FALSE)
	   YSide = (YSide == TRUE) ? FALSE : TRUE;

	 createNonPriorBox(AllAxes[i]);
	 gtk_box_pack_start(GTK_BOX(parent), AllAxes[i]->box, FALSE, FALSE, 0);
      }
   }
   updateStereoFrame();
}

static void 
changePriorPh(GtkSpinButton *spin, Axis *axis)
{
   gint i;
   gfloat dph;

   if(LastChange == Rotation)
   {
      LastChange = None;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(RotSpin), 0);
   }
   
   if(LastChange != ChangePh)
   {
      for(i = 0; i < 5; i++)
      {
	 AllAxes[i]->pre_th = *(AllAxes[i]->th);
	 AllAxes[i]->pre_ph = *(AllAxes[i]->ph);
      }
      LastChange = ChangePh;
   }

   setParamet(spin, axis->ph);
   dph = *(axis->ph) - axis->pre_ph;

   for(i = 0; i < 5; i++)
   {
      if(AllAxes[i] != axis)
      {
	 GtkWidget *parent = AllAxes[i]->box->parent;

	 *(AllAxes[i]->ph) = AllAxes[i]->pre_ph + dph;

	 if(*(AllAxes[i]->ph) > 360) *(AllAxes[i]->ph) -= 360;
	 if(*(AllAxes[i]->ph) < 0)   *(AllAxes[i]->ph) += 360;

	 createNonPriorBox(AllAxes[i]);
	 gtk_box_pack_start(GTK_BOX(parent), AllAxes[i]->box, FALSE, FALSE, 0);
      }
   }
	 
   updateStereoFrame();
}

static void 
rotateAroundPriorAxes(GtkSpinButton *spin, Axis *axis)
{
   gint i;
   gfloat mat[3][3];
   gfloat rot;
   gboolean normal;

   if(LastChange != Rotation)
   {
      for(i = 0; i < 5; i++)
      {
	 AllAxes[i]->pre_ph = *(AllAxes[i]->ph);
	 AllAxes[i]->pre_th = *(AllAxes[i]->th);
      }
      LastChange = Rotation;
   }
      
   rot = gtk_spin_button_get_value_as_float(spin);   
   get_rotate_matrix(axis->pre_th, axis->pre_ph, rot, mat);

   for(i = 0; i < 5; i++)
   {
      if(AllAxes[i] != axis)
      {
	 GtkWidget *parent = AllAxes[i]->box->parent;
	 
	 normal = rotate( AllAxes[i]->pre_th, AllAxes[i]->pre_ph, 
			 AllAxes[i]->th, AllAxes[i]->ph, mat);
	 
	 if(AllAxes[i] == &Y && normal == FALSE)
	   YSide = (YSide == TRUE) ? FALSE : TRUE;
	 
	 createNonPriorBox(AllAxes[i]);
	 gtk_box_pack_start(GTK_BOX(parent), AllAxes[i]->box, FALSE, FALSE, 0);
      }
   }
   updateStereoFrame();
}

static void 
PriorAxisChange(GtkRadioButton *radio, Axis *axis)
{
   GtkWidget *parent;

   if(axis == PriorAxis) return;

   parent = PriorAxis->box->parent;
   createNonPriorBox(PriorAxis);
   gtk_box_pack_start(GTK_BOX(parent), PriorAxis->box, FALSE, FALSE, 0);

   parent = axis->box->parent;
   createPriorBox(axis);
   gtk_box_pack_start(GTK_BOX(parent), axis->box, FALSE, FALSE, 0);

   PriorAxis = axis;

   gtk_spin_button_set_value(GTK_SPIN_BUTTON(RotSpin), 0);
   LastChange = None;
}

/*====================================================================*
* (Exported Procedure)                                                *
*                                                                     *
*                    changeOpticalAngle                               *
*=====================================================================*/ 
void 
changeOpticalAngle()
{
   gfloat V, V2, mat[3][3];
   gfloat ath, aph, bth, bph, xth, xph, zth, zph, yth, yph, anth, anph;
   Axis *anotherOpticAxis;
   GtkWidget *parent;
   gboolean normal;
   gint i;

   V2 = parameter.optic_axial_angle;
   V = V2/2;

   if(PriorAxis == &X || PriorAxis == &Y || PriorAxis == &Z) 
   {
      get_rotate_matrix(90, *(Y.ph)-90, *(Y.th), mat);
      rotate(*(Z.th), *(Z.ph), &zth, &zph, mat);

      ath = bth = 90;
      if(YSide == TRUE)
      {
	 aph = zph + V; bph = zph - V;
      }
      else
      {
	 aph = zph - V; bph = zph + V;
      }
	 
      get_rotate_matrix(90, *(Y.ph)-90, -*(Y.th), mat);
      rotate(ath, aph, A.th, A.ph, mat);
      rotate(bth, bph, B.th, B.ph, mat);
   }
   else
   {
      if(PriorAxis == &A) anotherOpticAxis = &B;
      else                anotherOpticAxis = &A;

      get_rotate_matrix(90, *(PriorAxis->ph)-90, *(PriorAxis->th), mat);

      normal = rotate(*(Y.th), *(Y.ph), &yth, &yph, mat);
      if( (normal==TRUE && YSide==FALSE) ||  (normal==FALSE && YSide==TRUE) )
 	 yph += 180;

      xth = 90 - V; zth = V; anth = V2;
      
      if(PriorAxis == &A)
      {
	 zph = anph = yph + 90;
	 xph = yph - 90;
      }
      else
      {
	 xph = yph + 90;
	 zph = anph = yph - 90;
      }
	 
      get_rotate_matrix(90, *(PriorAxis->ph)-90, -*(PriorAxis->th), mat);
      rotate(xth, xph, X.th, X.ph, mat);
      rotate(zth, zph, Z.th, Z.ph, mat);
      rotate(anth, anph, anotherOpticAxis->th, anotherOpticAxis->ph, mat);
   }

   for(i = 0; i < 5; i++)
   {
      parent = AllAxes[i]->box->parent;

      if(AllAxes[i] == PriorAxis)
	createPriorBox(AllAxes[i]);
      else
	createNonPriorBox(AllAxes[i]);

      gtk_box_pack_start(GTK_BOX(parent), AllAxes[i]->box, FALSE, FALSE, 0);
   }
   updateStereoFrame();
   LastChange = None;
}  
      
      

/*====================================================================*
* Rotational Procedure of orientation				      *
*          	     get_rotate_matrix                                * 
*                    rotate                                           *
*=====================================================================*/ 
static void  /* assume 3x3 matrix */
get_rotate_matrix(gfloat th0, gfloat ph0, gfloat d0, gfloat mat[][3])
{
   double v0[3], cos_d, sin_d;
   int i, j;

   th0 *= M_PI/180;
   ph0 *= M_PI/180;
   d0 *= M_PI/180;

   v0[0] = sin(th0)*cos(ph0);
   v0[1] = sin(th0)*sin(ph0);
   v0[2] = cos(th0);
   cos_d = cos(d0);
   sin_d = sin(d0);

   for(i = 0; i < 3; i++)
   {
      for(j=0; j < 3; j++)
      {
	 mat[i][j] = v0[i]*v0[j]*(1-cos_d);
	 if( i == j )
	   mat[i][j] += cos_d;
	 else if ( i == (j+1)%3 )
	   mat[i][j] += v0[(i+1)%3]*sin_d;
	 else
	   mat[i][j] -= v0[(j+1)%3]*sin_d;
      }
   }
}

static gboolean
rotate( gfloat th,  gfloat ph, gfloat* rth, gfloat* rph, gfloat mat[][3])
{
   double v[3], rv[3];
   int i, j;

   th *= M_PI/180;
   ph *= M_PI/180;

   v[0] = sin(th)*cos(ph);
   v[1] = sin(th)*sin(ph);
   v[2] = cos(th);

   for(i = 0; i < 3; i++)
   {
      rv[i] = 0;
      for(j = 0; j < 3; j++)
	rv[i] += mat[i][j]*v[j];
   }
 
   if(rv[2] > 1 ) 
   {
      *rth = 0; *rph = ph;
   }
   else if(rv[2] < -1) 
   {
      *rth = M_PI; *rph = ph;
   }
   else
   {
      *rth = acos(rv[2]);
      if(*rth < 1e-2 || fabs(M_PI - *rth) < 1e-2)
	*rph = ph;
      else
      {
	 *rph = acos(rv[0]/sqrt(rv[0]*rv[0] + rv[1]*rv[1]));
	 if(rv[1] < 0 ) *rph = 2*M_PI - *rph;
      }
   }

   *rth *= 180/M_PI;
   *rph *= 180/M_PI;

   if(*rth > 90 + 1e-2)
   {
      *rth = 180 - *rth;
      *rph += 180;
      if(*rph >= 360 ) *rph -= 360;
      return FALSE;
   }
   return TRUE;
}
   
