/*===========================================================================*
 * uniaxis.c                                                                 *
 *                                                                           *
 *      Procedures for calculating conoscopic light intensity                *
 *      (Uniaxial Conoscopic Figure)                                         *
 *                                                                           *
 * EXPORTED PROCEDURES:                                                      *
 *      void init_uniaxial(double thc, double phc,                           *
 *                         double omega, double epsilon, double d)           *
 *      void setup_uniaxial(double th, double ph)                            *
 *      double get_uniaxial_retardation()                                    *
 *      double get_uniaxial_diagonal_degree()                                *
 *                                                                           *
 *      (In these procedures angles are expressed as radian)                 *
 *                                                            (by A. Miyake) *
 *===========================================================================*/
#include <math.h>


/*==================*
 * STATIC VARIABLES *
 *==================*/
static double RI2;                  /* omega*epsilon */
static double RI2O;                 /* omega*omega */
static double RI2E;                 /* epsilon*epsilon */
static double RayLength;             /* Ray length passing through crystal */
static double thickness;

static double xc, yc, zc;           /* unit vector component of C axis */ 

static double cos_rthc, sin_rthc;          
static double cos_rphc, sin_rphc;          


/*====================* 
 * Internal Functions *
 *====================*/
static void rotate_optic_axis(double th, double ph);

/*===========================================================================*
 * (Exported Procedure)                                                      *
 *                                                                           *
 *            init_uniaxial                                                   *
 *               (Initial setting before calculating conoscopic figure)      *
 *===========================================================================*/
void 
init_uniaxial(double thc, double phc, double omega, double epsilon, double d)
{
   double sin_thc, cos_thc;     /* orientations of optic axes */
   double sin_phc, cos_phc;

   sin_thc = sin(thc); cos_thc = cos(thc);    
   sin_phc = sin(phc); cos_phc = cos(phc);    
   
   xc = sin_thc*cos_phc;  yc = sin_thc*sin_phc;  zc = cos_thc;
 
   RI2 = omega*epsilon;
   RI2O = omega*omega;
   RI2E = epsilon*epsilon;

   thickness = d;
}

/*==========================================================================*
* (Exported Procedure)                                                      *
*                                                                           *
*            setup_uniaxial                                                 *
*               (setting before calculating each ray interderence color)    *
*===========================================================================*/
void 
setup_uniaxial(double th, double ph)
{
   rotate_optic_axis(th, ph);
   RayLength = thickness/cos(th);
}

/*==========================================================================*
* (Exported Procedure)                                                      *
*                                                                           *
*                   get_uniaxial_retardation                                *
*===========================================================================*/
double 
get_uniaxial_retardation()
{
   double no = sqrt(RI2O);
   double ne = RI2/sqrt(RI2O*sin_rthc*sin_rthc + RI2E*cos_rthc*cos_rthc);

   return  RayLength* (ne - no);
}

/*==========================================================================*
* (Exported Procedure)                                                      *
*                                                                           *
*                   get_uniaxial_diagonal_degree                            *
*===========================================================================*/
double 
get_uniaxial_diagonal_degree()
{
   double dd = 2*sin_rphc*cos_rphc;

   if(dd > 1) return 1;
   else if( dd < -1) return -1;
   else return dd;
}

/*==========================================================================*
* (Internal Procedure)                                                      *
*                                                                           *
*                   rotate_optic_axis                                       *
*===========================================================================*/
static void 
rotate_optic_axis(double th, double ph) 
{
   double cos_th = cos(th);
   double sin_th = sin(th);
   double cos_ph = cos(ph);
   double sin_ph = sin(ph);

   double rxc, ryc, rzc;        /* unit vector component of rotated C */ 

   double k00, k10, k11, k20, k21, k22;

   k00 = cos_ph*cos_ph*cos_th + sin_ph*sin_ph;
   k10 = sin_ph*cos_ph*(cos_th-1);
   k11 = sin_ph*sin_ph*cos_th + cos_ph*cos_ph;
   k20 = cos_ph*sin_th;
   k21 = sin_ph*sin_th;
   k22 = cos_th;

   rxc = k00*xc + k10*yc - k20*zc;
   ryc = k10*xc + k11*yc - k21*zc;
   rzc = k20*xc + k21*yc + k22*zc;

   cos_rthc = rzc;
   sin_rthc = sqrt(1 - rzc*rzc);
   
   cos_rphc = rxc/sin_rthc;
   sin_rphc = ryc/sin_rthc;
}

